<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelMTCOperadorService {
    
	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
	}
    
	private function setParameters($conditions, $operador) {
    	return array (
            ':DTINI' => $conditions['DTENTRVENDA'][0], 
            ':DTFIN' => $conditions['DTENTRVENDA'][1], 
            ':CDFILIAL' => $operador['CDFILIAL'], 
            ':CDOPERADOR' => $operador['CDOPERADOR'], 
            ':MTCBASE' => $operador['MTCBASE'], 
            ':NRORG' => $conditions['NRORG'], 
            ':TCDPRODUTO' => $conditions['TCDPRODUTO'],
            ':CDPRODLIST' => $conditions['CDPRODLIST'],
            ':CDARVPRODF' => $conditions['CDARVPRODF'],
            ':CDARVPRODI' => $conditions['CDARVPRODI'],
            ':CDNVPRODUTO' => str_replace('%', '', $conditions['CDNVPRODUTO']),
            ':CDNVPRODUTOAGRUP' => $conditions['CDNVPRODUTOAGRUP'],
            ':PRODNIV' => $conditions['PRODNIV']
        );
    }
    
    private function paramsToMTCBase($conditions) {
    	return array (
            ':DTINI' => $conditions['DTENTRVENDA'][0], 
            ':DTFIN' => $conditions['DTENTRVENDA'][1], 
            ':CDOPERADOR' => implode('_',$conditions['CDOPERADOR']), 
            ':CDFILIAL' => implode('_',$conditions['CDFILIAL']), 
            ':TCDFILIAL' => count($conditions['CDFILIAL']) > 0 ? 'N' : 'T',
            ':TCDOPERADOR' => count($conditions['CDOPERADOR']) > 0 ? 'N' : 'T',
            ':NRORG' => $conditions['NRORG']
        );
        
    }
    
    public function getBase($conditions){
        $paramsBase = self::paramsToMTCBase($conditions);
        $arrayOper = $conditions['CDOPERADOR'];
        sort($arrayOper);
        $base = [];
        foreach ($arrayOper as $oper) {
            $estrutura = array (
                    'CDFILIAL' => null,
                    'CDOPERADOR' => $oper,
                    'MTCBASE' => null);
            $paramsBase['CDOPERADOR'] = $oper;
            $retorno = $this->entityManager->getConnection()->fetchAll(self::GET_MTCBASE_OPERADOR, $paramsBase);
            if(empty($retorno)){
                $base[] = $estrutura;
            } else {
                foreach ($retorno as $value) {
                    $base[] = $value;
                }
            }
        }
        
        $resultFinal = [];
        $operadorIndex = -1;
        $arrayOperExists = [];
        foreach ($base as $operador) {
            if(!array_key_exists($operador['CDOPERADOR'], $arrayOperExists)){
                $operadorIndex ++;
            }
            $valueOfResult = self::getData($conditions, $operador, $operadorIndex);
            array_push($resultFinal, $valueOfResult);
            $arrayOperExists[$operador['CDOPERADOR']] = false;
        };
        
        return $resultFinal;
    }
    
    public function getData($conditions, $operador, $operadorIndex){
        $params = self::setParameters($conditions, $operador);
        $query = self::INI_QUERY_GET_MTC_PROD_FILIAL."OPERADORMTC".$operadorIndex.self::INI2_QUERY_GET_MTC_PROD_FILIAL."OPERADORQTD".$operadorIndex.self::INI3_QUERY_GET_MTC_PROD_FILIAL."OPERADORVR".$operadorIndex.self::FIM_QUERY_GET_MTC_PROD_FILIAL; 
        $query = strval(trim($query));
        $dados = $this->entityManager->getConnection()->fetchAll($query, $params);
        
        return $dados;
    }

    const INI_QUERY_GET_MTC_PROD_FILIAL = "SELECT FILOCAVE.CDFILIAL, FILOCAVE.NMFILIAL, FILOCAVE.CDOPERADOR,  SUBSTR(FILOCAVE.NMOPERADOR, 1 ,INSTR(FILOCAVE.NMOPERADOR, ' ')) AS NMOPERADOR, FILOCAVE.CDFILIAL||' - '||FILOCAVE.NMFILIAL AS LABELFILIAL,
    PV.CDARVPROD||' - '||PV.NMPRODUTO AS LABELPRODAVO, P.CDARVPROD||FILOCAVE.CDFILIAL AS INDEXPRODUTO, P.CDARVPROD||' - '||P.NMPRODUTO AS LABELPROD, P.CDARVPROD, P.SGUNIDADE, I.CDPRODUTO, P.NMPRODUTO, ((SUM(I.QTPRODVEND)/:MTCBASE)*1000) AS ";
    
    const INI2_QUERY_GET_MTC_PROD_FILIAL = ", SUM(I.QTPRODVEND) AS  ";
    
    const INI3_QUERY_GET_MTC_PROD_FILIAL = ", SUM(TRUNC(((I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0))) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)),2)) AS  ";
    
    const FIM_QUERY_GET_MTC_PROD_FILIAL = "
          FROM ITEMVENDA I
          JOIN PRODUTO P
            ON I.CDPRODUTO = P.CDPRODUTO
          JOIN PRODUTO PV
            ON P.NRORG = PV.NRORG
          JOIN (SELECT V.IDSITUVENDA, V.NRSEQVENDA, FILOCA.CDCAIXA, FILOCA.NMCAIXA, FILOCA.CDFILIAL, FILOCA.NMFILIAL,
                       V.DTENTRVENDA, OP.NMOPERADOR, OP.CDOPERADOR
                 FROM VENDA V
                 JOIN (SELECT FLC.CDCAIXA, FLC.NMCAIXA, FLC.NMFILIAL, FLC.CDFILIAL, FLC.CDLOJA, FLC.NMLOJA
                         FROM V_FILIALLOJACAIXA FLC
                         WHERE FLC.CDFILIAL = :CDFILIAL
                        ) FILOCA 
                   ON V.CDFILIAL = FILOCA.CDFILIAL
                  AND V.CDCAIXA = FILOCA.CDCAIXA
                  AND V.CDLOJA = FILOCA.CDLOJA
                  JOIN OPERADOR OP 
                  ON V.CDOPERADOR = OP.CDOPERADOR
                WHERE V.IDSITUVENDA = 'O' 
                  AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN 
                  AND V.IDORIGEMVENDA <> 'VND_MAN'
                  AND OP.CDOPERADOR = :CDOPERADOR ) FILOCAVE
            ON I.CDFILIAL = FILOCAVE.CDFILIAL 
           AND I.NRSEQVENDA = FILOCAVE.NRSEQVENDA
           AND I.CDCAIXA = FILOCAVE.CDCAIXA
         WHERE P.CDPRODUTO LIKE PV.CDPRODUTO||'%'
           AND PV.CDNVPRODUTO = :CDNVPRODUTOAGRUP
           AND ((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0 ) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (P.CDARVPROD LIKE :PRODNIV) OR (:TCDPRODUTO = 'T')) 
         GROUP BY  FILOCAVE.CDFILIAL, FILOCAVE.NMFILIAL, P.CDARVPROD, I.CDPRODUTO, P.NMPRODUTO, PV.CDARVPROD, PV.NMPRODUTO, P.SGUNIDADE, FILOCAVE.CDOPERADOR, FILOCAVE.NMOPERADOR
         ORDER BY FILOCAVE.CDOPERADOR, I.CDPRODUTO
    ";

    const GET_MTCBASE_OPERADOR = "
        SELECT V.CDFILIAL, V.CDOPERADOR, COUNT(V.NRSEQVENDA) AS MTCBASE
            FROM VENDA V 
            WHERE V.IDSITUVENDA = 'O' 
               AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN 
               AND ( ( INSTR( :CDFILIAL, V.CDFILIAL ) <> 0 ) OR ( :TCDFILIAL = 'T' ) ) 
               AND ( ( INSTR( :CDOPERADOR, V.CDOPERADOR ) <> 0 ) OR ( :TCDOPERADOR = 'T' ) )
            GROUP BY V.CDFILIAL, V.CDOPERADOR
    ";
}